#!/usr/bin/env python

from distutils.core import setup

setup(
    name="sauceconnect",
    version="0.0.1",
    description="Python bindings for libsauceconnect",
    author="Sauce Labs",
    author_email="info@saucelabs.com",
    url="http://saucelabs.com/",
    packages=["sauceconnect"],
)
